/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2013 Jean-Sebastien Lerat (Jean-Sebastien.Lerat@ulb.ac.be)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

/**
 *
 * @author Jean-Sebastien Lerat (Jean-Sebastien.Lerat@ulb.ac.be)
 * @version 1.00, 24/06/2013
 * Native implementation of the measure library for CoNet (http://psbweb05.psb.ugent.be/conet/)
 */

#ifndef __RENORMALIZER_H__
	#define __RENORMALIZER_H__

#include "jniutils.h"
#include <stdlib.h>
#include <math.h>
#include <string.h>


#define DEFAULT_RENORMALIZER_CLASS	"be/ac/ulb/mlg/utils/DefaultRenormalizer"
#define ABSTRACT_RENORMALIZER		"Lbe/ac/ulb/mlg/utils/Renormalizer;"
#define SUM_NORMILIZER				0.0
#define RAND_INT(max)				(rand() % max)

#define property_enabled(env,klass,object,method) (*env)->CallBooleanMethod(env,object,(*env)->GetMethodID(env, klass, method,"()Z"))

typedef struct DefaultRenormalizerConfig{
	int groupCount;
	jsize M;
	jboolean pairShuffle;
	jint* groups;
	int * specificSize;
	jdouble ** specificNormalizer;
	jdouble ** sum;
	jint* features;
} DefaultRenormalizerConfig;

void free_renormalizer_config(JNIEnv *env, DefaultRenormalizerConfig *);
DefaultRenormalizerConfig * defaultRenormalizer(JNIEnv * env,jobject renormalizer,jdouble * data[],const jsize N,const jsize M);
void updateRenormalization(DefaultRenormalizerConfig * config,double * vect,const int row, const int ignore);
//void simulateRenormalization(DefaultRenormalizerConfig * config,double * vect,const int row);
int features_contains(jint* features, const int row);
void simulateRenormalization(DefaultRenormalizerConfig *,jdouble * shuffledFirst,jdouble * shuffledSecond,const int first,const int second);
void inline sumCol(jdouble * row,jdouble * sum,jsize N);
void inline diffCol(jdouble * row,jdouble * sum,jsize N);
void inline mergeRenormilizer(DefaultRenormalizerConfig *);

#endif
